import decimal
import typing

from authorizenet import apicontractsv1
from django.conf import settings
from django.contrib.auth.mixins import LoginRequiredMixin
from django.core.exceptions import ValidationError
from django.db import transaction
from django.http import HttpRequest, HttpResponse, HttpResponseRedirect
from django.urls import reverse
from django.utils import timezone
from django.utils.decorators import method_decorator
from django.utils.translation import gettext_lazy as _
from django.views.decorators.cache import cache_control, cache_page
from django.views.generic import FormView, RedirectView, TemplateView
from terminusgps.authorizenet.service import (
    AuthorizenetControllerExecutionError,
)
from terminusgps.mixins import HtmxTemplateResponseMixin
from terminusgps_payments.models import (
    AddressProfile,
    PaymentProfile,
    Subscription,
)
from terminusgps_payments.services import AuthorizenetService

from terminusgps_notifications import services
from terminusgps_notifications.forms import CustomerSubscriptionCreationForm
from terminusgps_notifications.models import WialonToken


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
@method_decorator(cache_control(private=True), name="dispatch")
class DashboardView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, TemplateView
):
    content_type = "text/html"
    extra_context = {
        "title": "Dashboard",
        "subtitle": "We know where ours are... do you?",
    }
    http_method_names = ["get"]
    partial_template_name = (
        "terminusgps_notifications/customers/partials/_dashboard.html"
    )
    template_name = "terminusgps_notifications/customers/dashboard.html"

    def setup(self, request: HttpRequest, *args, **kwargs) -> None:
        """Adds :py:attr:`customer`, :py:attr:`has_token` and :py:attr:`has_subscription` to the view."""
        customer = (
            services.get_customer(request.user)
            if hasattr(request, "user")
            else None
        )
        token = (
            services.get_wialon_token(request.user)
            if hasattr(request, "user")
            else None
        )
        subscription = (
            getattr(customer, "subscription")
            if customer is not None and hasattr(customer, "subscription")
            else None
        )
        self.customer = customer
        self.has_token = bool(token)
        self.has_subscription = bool(subscription)
        return super().setup(request, *args, **kwargs)

    def get_context_data(self, **kwargs) -> dict[str, typing.Any]:
        """Adds :py:attr:`customer`, :py:attr:`has_token` and :py:attr:`has_subscription` to the view context."""
        context: dict[str, typing.Any] = super().get_context_data(**kwargs)
        context["customer"] = self.customer
        context["has_token"] = self.has_token
        context["has_subscription"] = self.has_subscription
        return context


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
@method_decorator(cache_control(private=True), name="dispatch")
class AccountView(LoginRequiredMixin, HtmxTemplateResponseMixin, TemplateView):
    content_type = "text/html"
    extra_context = {"title": "Account"}
    http_method_names = ["get"]
    partial_template_name = (
        "terminusgps_notifications/customers/partials/_account.html"
    )
    template_name = "terminusgps_notifications/customers/account.html"

    def setup(self, request: HttpRequest, *args, **kwargs) -> None:
        """Adds :py:attr:`customer`, :py:attr:`has_token` and :py:attr:`login_params` to the view."""
        customer = (
            services.get_customer(request.user)
            if hasattr(request, "user")
            else None
        )
        token = (
            services.get_wialon_token(request.user)
            if hasattr(request, "user")
            else None
        )
        login_params = (
            services.get_wialon_login_parameters(request.user)
            if hasattr(request, "user")
            else None
        )
        self.customer = customer
        self.has_token = bool(token)
        self.login_params = login_params
        return super().setup(request, *args, **kwargs)

    @transaction.atomic
    def get(self, request: HttpRequest, *args, **kwargs) -> HttpResponse:
        """Saves a Wialon API token for the user based on the ``user_name`` and ``access_token`` path parameters."""
        if hasattr(request, "user") and self.customer is not None:
            user = getattr(request, "user")
            username = request.GET.get("user_name")
            access_token = request.GET.get("access_token")
            if all(
                [
                    username,
                    access_token,
                    hasattr(user, "username"),
                    getattr(user, "username") == self.customer.user.username,
                ]
            ):
                if hasattr(self.customer, "token"):
                    old_token = getattr(self.customer, "token")
                    old_token.delete()
                new_token = WialonToken()
                new_token.name = access_token
                new_token.customer = self.customer
                new_token.save()
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs) -> dict[str, typing.Any]:
        """Adds :py:attr:`customer`, :py:attr:`has_token` and :py:attr:`login_params` to the view context."""
        context: dict[str, typing.Any] = super().get_context_data(**kwargs)
        context["customer"] = self.customer
        context["has_token"] = self.has_token
        context["login_params"] = self.login_params
        return context


class SubscriptionView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, TemplateView
):
    content_type = "text/html"
    extra_context = {"title": "Subscription"}
    http_method_names = ["get"]
    partial_template_name = (
        "terminusgps_notifications/customers/partials/_subscription.html"
    )
    template_name = "terminusgps_notifications/customers/subscription.html"

    def setup(self, request: HttpRequest, *args, **kwargs) -> None:
        """Adds :py:attr:`customer` to the view."""
        customer = (
            services.get_customer(request.user)
            if hasattr(request, "user")
            else None
        )
        self.customer = customer
        return super().setup(request, *args, **kwargs)

    def get_context_data(self, **kwargs) -> dict[str, typing.Any]:
        """Adds :py:attr:`customer` to the view context."""
        context: dict[str, typing.Any] = super().get_context_data(**kwargs)
        context["customer"] = self.customer
        return context


class NotificationsView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, TemplateView
):
    content_type = "text/html"
    extra_context = {"title": "Notifications"}
    http_method_names = ["get"]
    partial_template_name = (
        "terminusgps_notifications/customers/partials/_notifications.html"
    )
    template_name = "terminusgps_notifications/customers/notifications.html"

    def setup(self, request: HttpRequest, *args, **kwargs) -> None:
        """Adds :py:attr:`customer` and :py:attr:`has_token` to the view context."""
        customer = (
            services.get_customer(request.user)
            if hasattr(request, "user")
            else None
        )
        token = (
            services.get_wialon_token(request.user)
            if hasattr(request, "user")
            else None
        )
        packages = (
            getattr(customer, "package").all()
            if customer and hasattr(customer, "package")
            else None
        )

        self.customer = customer
        self.has_token = bool(token)
        self.packages = packages
        return super().setup(request, *args, **kwargs)

    def get_context_data(self, **kwargs) -> dict[str, typing.Any]:
        """Adds :py:attr:`customer` and :py:attr:`has_token` to the view context."""
        context: dict[str, typing.Any] = super().get_context_data(**kwargs)
        context["customer"] = self.customer
        context["has_token"] = self.has_token
        context["packages"] = self.packages
        return context


class CustomerSubscriptionCreateView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, FormView
):
    content_type = "text/html"
    extra_context = {
        "title": "Create Subscription",
        "setup_fee": settings.NOTIFICATIONS_SETUP_FEE,
    }
    form_class = CustomerSubscriptionCreationForm
    http_method_names = ["get", "post"]
    partial_template_name = "terminusgps_notifications/customers/partials/_create_subscription.html"
    template_name = (
        "terminusgps_notifications/customers/create_subscription.html"
    )

    def setup(self, request: HttpRequest, *args, **kwargs) -> None:
        """Adds :py:attr:`customer` and :py:attr:`anet_service` to the view."""
        customer = (
            services.get_customer(request.user)
            if hasattr(request, "user")
            else None
        )
        self.customer = customer
        self.anet_service = AuthorizenetService()
        return super().setup(request, *args, **kwargs)

    def get_context_data(self, **kwargs) -> dict[str, typing.Any]:
        """Adds :py:attr:`customer` to the view context."""
        context: dict[str, typing.Any] = super().get_context_data(**kwargs)
        context["customer"] = self.customer
        context["total"] = self.customer.subtotal + context["setup_fee"]
        return context

    def get_form(self, form_class=None) -> CustomerSubscriptionCreationForm:
        """Sets customer payment profiles and address profiles in the form."""
        form = super().get_form(form_class=form_class)
        payment_qs = PaymentProfile.objects.for_user(self.request.user)
        address_qs = AddressProfile.objects.for_user(self.request.user)
        pprofile_field = form.fields["payment_profile"]
        aprofile_field = form.fields["address_profile"]
        pprofile_field.queryset = payment_qs
        pprofile_field.empty_label = None
        aprofile_field.queryset = address_qs
        aprofile_field.empty_label = None
        return form

    @transaction.atomic
    def form_valid(
        self, form: CustomerSubscriptionCreationForm
    ) -> HttpResponse:
        """Creates a subscription for the customer in Authorizenet."""
        if self.customer is None:
            form.add_error(
                None,
                ValidationError(
                    _(
                        "Whoops! Your account doesn't have an associated customer. Please try again later"
                    ),
                    code="invalid",
                ),
            )
            return self.form_invalid(form=form)
        customer_profile = services.get_customer_profile(self.request.user)
        if customer_profile is None:
            form.add_error(
                None,
                ValidationError(
                    _(
                        "Whoops! Your account doesn't have an associated customer profile. Please try again later"
                    ),
                    code="invalid",
                ),
            )
            return self.form_invalid(form=form)

        # Set subscription variables
        payment_profile = form.cleaned_data["payment_profile"]
        address_profile = form.cleaned_data["address_profile"]
        start_date = timezone.now()
        name = "Terminus GPS Notifications"
        amount = self.customer.subtotal
        trial_amount = decimal.Decimal("0.00")
        setup_fee = settings.NOTIFICATIONS_SETUP_FEE

        # Set once-per-month interval
        interval = apicontractsv1.paymentScheduleTypeInterval()
        interval.length = 1
        interval.unit = apicontractsv1.ARBSubscriptionUnitEnum.months

        # Set infinitely recurring payment schedule
        schedule = apicontractsv1.paymentScheduleType()
        schedule.startDate = start_date
        schedule.totalOccurrences = 9999
        schedule.trialOccurrences = 1
        schedule.interval = interval

        # Set customer payment information
        profile = apicontractsv1.customerProfileIdType()
        profile.customerProfileId = str(customer_profile.pk)
        profile.customerPaymentProfileId = str(payment_profile.pk)
        profile.customerAddressId = str(address_profile.pk)

        # Set Authorizenet subscription contract
        contract = apicontractsv1.ARBSubscriptionType()
        contract.name = name
        contract.amount = amount
        contract.trialAmount = trial_amount
        contract.profile = profile
        contract.paymentSchedule = schedule

        line_item_1 = apicontractsv1.lineItemType()
        line_item_1.itemId = "1"
        line_item_1.name = "Setup Fee"
        line_item_1.description = "One-time setup fee"
        line_item_1.quantity = "1"
        line_item_1.unitPrice = setup_fee

        line_item_2 = apicontractsv1.lineItemType()
        line_item_2.itemId = "2"
        line_item_2.name = "First Month"
        line_item_2.description = "First month of service"
        line_item_2.quantity = "1"
        line_item_2.unitPrice = amount

        line_items = apicontractsv1.ArrayOfLineItem()
        line_items.lineItem.append(line_item_1)
        line_items.lineItem.append(line_item_2)

        try:
            # Collect setup fee + month 1
            self.anet_service.charge_customer_profile(
                customer_profile,
                payment_profile,
                amount + setup_fee,
                line_items,
            )

            # Create subscription locally and in Authorizenet
            subscription = Subscription(
                name=name,
                amount=amount,
                customer_profile=customer_profile,
                payment_profile=payment_profile,
                address_profile=address_profile,
            )
            anet_response = self.anet_service.create_subscription(
                subscription, contract
            )
            subscription.pk = int(anet_response.subscriptionId)
            subscription.save()
            self.customer.subscription = subscription
            self.customer.save()
            return HttpResponseRedirect(
                reverse("terminusgps_notifications:subscription")
            )
        except AuthorizenetControllerExecutionError as e:
            match e.code:
                case _:
                    form.add_error(
                        None,
                        ValidationError(
                            _("Whoops! %(error)s"),
                            code="invalid",
                            params={"error": str(e)},
                        ),
                    )
            return self.form_invalid(form=form)


class CustomerStatsView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, TemplateView
):
    content_type = "text/html"
    http_method_names = ["get"]
    partial_template_name = (
        "terminusgps_notifications/customers/partials/_stats.html"
    )
    template_name = "terminusgps_notifications/customers/stats.html"

    def setup(self, request: HttpRequest, *args, **kwargs) -> None:
        """Adds :py:attr:`customer` and :py:attr:`has_token` to the view."""
        customer = (
            services.get_customer(request.user)
            if hasattr(request, "user")
            else None
        )
        token = (
            services.get_wialon_token(request.user)
            if hasattr(request, "user")
            else None
        )
        self.customer = customer
        self.has_token = bool(token)
        return super().setup(request, *args, **kwargs)

    def get_context_data(self, **kwargs) -> dict[str, typing.Any]:
        """Adds :py:attr:`customer` and :py:attr:`has_token` to the view context."""
        context: dict[str, typing.Any] = super().get_context_data(**kwargs)
        context["customer"] = self.customer
        context["has_token"] = self.has_token
        return context


class WialonLoginView(LoginRequiredMixin, RedirectView):
    http_method_names = ["get"]
    permanent = True
    query_string = True
    url = "https://hosting.terminusgps.com/login.html"



"""
Django settings for src project.

Generated by 'django-admin startproject' using Django 5.2.6.

For more information on this file, see
https://docs.djangoproject.com/en/5.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/5.2/ref/settings/
"""

import base64
import decimal
import os
import sys
from pathlib import Path

from terminusgps.authorizenet.constants import Environment, ValidationMode
from terminusgps.wialon.flags import TokenFlag

ALLOWED_HOSTS = ["127.0.0.1", "localhost"]
BASE_DIR = Path(__file__).resolve().parent.parent
LOGIN_REDIRECT_URL = "/dashboard/"
LOGIN_URL = "/login/"
DEBUG = True
DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"
DJANGO_ENCRYPTED_FIELD_ALGORITHM = os.getenv(
    "DJANGO_ENCRYPTED_FIELD_ALGORITHM", "SS20"
)
DJANGO_ENCRYPTED_FIELD_KEY = base64.b64decode(
    os.getenv("DJANGO_ENCRYPTED_FIELD_KEY", "")
)
LANGUAGE_CODE = "en-us"
MERCHANT_AUTH_ENVIRONMENT = Environment.SANDBOX
MERCHANT_AUTH_LOGIN_ID = os.getenv("MERCHANT_AUTH_LOGIN_ID")
MERCHANT_AUTH_TRANSACTION_KEY = os.getenv("MERCHANT_AUTH_TRANSACTION_KEY")
MERCHANT_AUTH_VALIDATION_MODE = ValidationMode.TEST
ROOT_URLCONF = "src.urls"
SECRET_KEY = (
    "django-insecure-o63#9m5p-j-&o2ub897i9r+1#)81j8%e06-%-6u1ktsdw+*%+*"
)
STATIC_URL = "static/"
TIME_ZONE = "US/Central"
USE_I18N = False
USE_TZ = True
WIALON_RESOURCE_NAME = "Terminus GPS Notifications"
WIALON_TOKEN = os.getenv("WIALON_TOKEN")
WIALON_TOKEN_ACCESS_TYPE = (
    TokenFlag.VIEW_ACCESS
    | TokenFlag.MANAGE_NONSENSITIVE
    | TokenFlag.MANAGE_SENSITIVE
)
WSGI_APPLICATION = "src.wsgi.application"
NOTIFICATIONS_SETUP_FEE = decimal.Decimal("140.00")
EMAIL_BACKEND = "django.core.mail.backends.console.EmailBackend"

ADMINS = (
    ("Peter", "pspeckman@terminusgps.com"),
    ("Blake", "blake@terminusgps.com"),
    ("Lili", "lili@terminusgps.com"),
)

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False if "test" not in sys.argv else True,
    "formatters": {
        "verbose": {
            "format": "%(asctime)s [%(process)d] [%(module)s] [%(levelname)s] %(message)s",
            "datefmt": "[%Y-%m-%d %H:%M:%S%z]",
            "class": "logging.Formatter",
        },
        "simple": {
            "format": "%(asctime)s [%(levelname)s] %(message)s",
            "datefmt": "[%Y-%m-%d %H:%M:%S%z]",
            "class": "logging.Formatter",
        },
    },
    "handlers": {
        "console": {"class": "logging.StreamHandler", "formatter": "simple"},
        "console_verbose": {
            "class": "logging.StreamHandler",
            "formatter": "verbose",
        },
    },
    "root": {"handlers": ["console"], "level": "DEBUG"},
    "loggers": {
        "django": {
            "handlers": ["console"],
            "level": os.getenv("DJANGO_LOG_LEVEL", "INFO"),
            "propagate": False,
        },
        "terminusgps_notifications": {
            "handlers": ["console_verbose"],
            "level": os.getenv("NOTIFICATIONS_LOG_LEVEL", "DEBUG"),
            "propagate": False,
        },
        "terminusgps_payments": {
            "handlers": ["console_verbose"],
            "level": os.getenv("PAYMENTS_LOG_LEVEL", "DEBUG"),
            "propagate": False,
        },
        "authorizenet.sdk": {
            "handlers": ["console"],
            "level": os.getenv("AUTHORIZENET_LOG_LEVEL", "DEBUG"),
            "propagate": False,
        },
    },
}

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django_rq",
    "django_tasks",
    "terminusgps_payments.apps.TerminusgpsPaymentsConfig",
    "terminusgps_notifications.apps.TerminusgpsNotificationsConfig",
]

RQ_QUEUES = {"default": {"HOST": "localhost", "PORT": 6379, "DB": 0}}

TASKS = {
    "default": {
        "BACKEND": "django_tasks.backends.rq.RQBackend",
        "QUEUES": ["default"],
    }
}

CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.redis.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379",
    }
}

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": BASE_DIR / "db.sqlite3",
    }
}

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator"
    },
    {"NAME": "django.contrib.auth.password_validation.MinimumLengthValidator"},
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator"
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator"
    },
]



from django.contrib.auth import get_user_model
from django.test import TestCase, override_settings
from terminusgps.authorizenet.constants import Environment, ValidationMode

from terminusgps_notifications import models


@override_settings(
    MERCHANT_AUTH_ENVIRONMENT=Environment.SANDBOX,
    MERCHANT_AUTH_VALIDATION_MODE=ValidationMode.TEST,
)
class TerminusgpsNotificationsCustomerTestCase(TestCase):
    def setUp(self) -> None:
        self.test_company = "Test Company"
        self.test_creds = {
            "username": "testuser@testdomain.com",
            "email": "testuser@testdomain.com",
            "password": "super_secure_password1!",
        }
        self.test_user = get_user_model().objects.create_user(
            username=self.test_creds["username"],
            email=self.test_creds["email"],
            password=self.test_creds["password"],
        )
        self.test_customer = (
            models.TerminusgpsNotificationsCustomer.objects.create(
                user=self.test_user, company=self.test_company
            )
        )

    def tearDown(self) -> None:
        self.test_user.delete()

    def test_str(self) -> None:
        """Fails if the result of calling :py:func:`str` on the customer isn't equal to the user's username."""
        self.assertEqual(str(self.test_customer), str(self.test_user.username))

    def test_tax(self) -> None:
        """Fails if the automatically generated tax amount was incorrectly calculated."""
        self.test_customer.refresh_from_db()
        expected_tax = (
            self.test_customer.subtotal * (self.test_customer.tax_rate + 1)
            - self.test_customer.subtotal
        )
        self.assertAlmostEqual(
            float(self.test_customer.tax), float(expected_tax), places=2
        )

    def test_grand_total(self) -> None:
        """Fails if the automatically generated grand total amount was incorrectly calculated."""
        self.test_customer.refresh_from_db()
        expected_grand_total = self.test_customer.subtotal * (
            self.test_customer.tax_rate + 1
        )
        self.assertAlmostEqual(
            float(self.test_customer.grand_total),
            float(expected_grand_total),
            places=2,
        )


class WialonNotificationTestCase(TestCase):
    def setUp(self) -> None:
        self.test_creds = {
            "username": "testuser@testdomain.com",
            "email": "testuser@testdomain.com",
            "password": "super_secure_password1!",
        }
        self.test_user = get_user_model().objects.create_user(
            username=self.test_creds["username"],
            email=self.test_creds["email"],
            password=self.test_creds["password"],
        )
        self.test_customer = (
            models.TerminusgpsNotificationsCustomer.objects.create(
                user=self.test_user
            )
        )
        self.test_notification = models.WialonNotification(
            customer=self.test_customer,
            name="Test Notification",
            message="%NOTIFICATION%",
            method="sms",
            unit_list=[12345678, 56781234],
            trigger={
                "trg": {
                    "t": "sensor_value",
                    "p": {
                        "lower_bound": "-1.0",
                        "merge": 0,
                        "prev_msg_diff": 0,
                        "sensor_name_mask": "*IGN*",
                        "sensor_type": "engine operation",
                        "type": 0,
                        "upper_bound": "1.0",
                    },
                }
            },
        )
        self.test_notification.actions = self.test_notification.get_actions()
        self.test_notification.text = self.test_notification.get_text()
        # TODO: Create notification in Wialon before testing

    def tearDown(self) -> None:
        self.test_user.delete()



import json
import logging
import typing

from django.contrib.auth.mixins import LoginRequiredMixin
from django.core.exceptions import ValidationError
from django.db import transaction
from django.db.models import QuerySet
from django.forms import Form
from django.http import HttpRequest, HttpResponse, HttpResponseRedirect
from django.urls import reverse, reverse_lazy
from django.utils.decorators import method_decorator
from django.utils.translation import gettext_lazy as _
from django.views.decorators.cache import cache_control, cache_page
from django.views.generic import (
    CreateView,
    DeleteView,
    DetailView,
    FormView,
    ListView,
    TemplateView,
    UpdateView,
)
from terminusgps.mixins import HtmxTemplateResponseMixin
from terminusgps.wialon.session import WialonAPIError, WialonSession

from terminusgps_notifications import constants, forms, models, services

logger = logging.getLogger(__name__)


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
@method_decorator(cache_control(private=True), name="dispatch")
class WialonNotificationTriggerFormView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, FormView
):
    content_type = "text/html"
    form_class = forms.TriggerForm
    http_method_names = ["get", "post"]
    partial_template_name = (
        "terminusgps_notifications/triggers/partials/_form.html"
    )
    success_url = reverse_lazy(
        "terminusgps_notifications:create notifications"
    )
    template_name = "terminusgps_notifications/triggers/form.html"

    def get_initial(self, **kwargs) -> dict[str, typing.Any]:
        initial: dict[str, typing.Any] = super().get_initial(**kwargs)
        initial["unit_list"] = self.request.GET.getlist("unit_list")
        initial["resource_id"] = self.request.GET.get("resource_id")
        return initial

    def form_valid(self, form: forms.TriggerForm) -> HttpResponse:
        return HttpResponseRedirect(
            reverse(
                "terminusgps_notifications:create notifications",
                query={
                    "trigger_type": form.cleaned_data["t"],
                    "trigger_parameters": json.dumps(form.cleaned_data["p"]),
                    "resource_id": form.cleaned_data["resource_id"],
                    "unit_list": form.cleaned_data["unit_list"],
                },
            )
        )


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
@method_decorator(cache_control(private=True), name="dispatch")
class WialonNotificationTriggerParametersFormView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, FormView
):
    content_type = "text/html"
    http_method_names = ["get", "post"]
    partial_template_name = (
        "terminusgps_notifications/triggers/partials/_parameters_form.html"
    )
    success_url = reverse_lazy(
        "terminusgps_notifications:trigger parameters success"
    )
    template_name = "terminusgps_notifications/triggers/parameters_form.html"

    def get_form_class(self):
        if self.request.GET.get("t"):
            trigger_type = self.request.GET["t"]
        elif self.request.POST.get("t"):
            trigger_type = self.request.POST["t"]
        return forms.TRIGGER_FORM_MAP.get(trigger_type)

    def form_valid(self, form):
        return HttpResponseRedirect(
            reverse(
                "terminusgps_notifications:trigger parameters success",
                query={"p": json.dumps(form.cleaned_data)},
            )
        )


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
@method_decorator(cache_control(private=True), name="dispatch")
class WialonNotificationTriggerParametersSuccessView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, TemplateView
):
    content_type = "text/html"
    http_method_names = ["get"]
    partial_template_name = (
        "terminusgps_notifications/triggers/partials/_parameters_success.html"
    )
    template_name = (
        "terminusgps_notifications/triggers/parameters_success.html"
    )

    def get_context_data(self, **kwargs) -> dict[str, typing.Any]:
        context: dict[str, typing.Any] = super().get_context_data(**kwargs)
        context["value"] = self.request.GET.get("p", "{}")
        return context


@method_decorator(cache_page(timeout=60 * 15), name="get")
@method_decorator(cache_control(private=True), name="get")
class WialonNotificationUnitSelectFormView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, FormView
):
    content_type = "text/html"
    extra_context = {"title": "Select Units"}
    form_class = forms.WialonUnitSelectForm
    http_method_names = ["get", "post"]
    partial_template_name = (
        "terminusgps_notifications/units/partials/_form.html"
    )
    template_name = "terminusgps_notifications/units/form.html"
    success_url = reverse_lazy("terminusgps_notifications:triggers form")

    def get(self, request: HttpRequest, *args, **kwargs) -> HttpResponse:
        if "resource_id" not in request.GET or "items_type" not in request.GET:
            return HttpResponse(status=406)
        return super().get(request, *args, **kwargs)

    def post(self, request: HttpRequest, *args, **kwargs) -> HttpResponse:
        if (
            "resource_id" not in request.POST
            or "items_type" not in request.POST
        ):
            return HttpResponse(status=406)
        return super().post(request, *args, **kwargs)

    def get_form(self, form_class=None) -> forms.WialonUnitSelectForm:
        form = super().get_form(form_class=form_class)
        customer = services.get_customer(self.request.user)
        token = services.get_wialon_token(self.request.user)
        if customer and token:
            with WialonSession(token=token) as session:
                resource_id = self.request.GET.get(
                    "resource_id", self.request.POST.get("resource_id")
                )
                items_type = self.request.GET.get(
                    "items_type", self.request.POST.get("items_type")
                )
                form.fields["resource_id"].choices = [
                    (int(resource["id"]), str(resource["nm"]))
                    for resource in customer.get_resources_from_wialon(session)
                ]
                form.fields["unit_list"].choices = [
                    (int(unit["id"]), str(unit["nm"]))
                    for unit in customer.get_units_from_wialon(
                        resource_id=resource_id,
                        session=session,
                        items_type=items_type,
                    )
                ]
        return form

    def form_valid(self, form: forms.WialonUnitSelectForm) -> HttpResponse:
        return HttpResponseRedirect(
            reverse(
                "terminusgps_notifications:triggers form",
                query={
                    "resource_id": form.cleaned_data["resource_id"],
                    "unit_list": form.cleaned_data["unit_list"],
                },
            )
        )


class WialonNotificationCreateSuccessView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, TemplateView
):
    content_type = "text/html"
    extra_context = {"title": "Created Notification"}
    http_method_names = ["get"]
    partial_template_name = (
        "terminusgps_notifications/notifications/partials/_create_success.html"
    )
    template_name = (
        "terminusgps_notifications/notifications/create_success.html"
    )


class WialonNotificationCreateView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, CreateView
):
    extra_context = {
        "title": "Create Notification",
        "message_tags": constants.WialonNotificationMessageTag.choices,
    }
    content_type = "text/html"
    form_class = forms.WialonNotificationCreationForm
    http_method_names = ["get", "post"]
    model = models.WialonNotification
    partial_template_name = (
        "terminusgps_notifications/notifications/partials/_create.html"
    )
    success_url = reverse_lazy(
        "terminusgps_notifications:create notifications success"
    )
    template_name = "terminusgps_notifications/notifications/create.html"

    def get_initial(self, **kwargs) -> dict[str, typing.Any]:
        initial: dict[str, typing.Any] = super().get_initial(**kwargs)
        schedule = {"f1": 0, "f2": 0, "t1": 0, "t2": 0, "m": 0, "w": 0, "y": 0}
        initial["timezone"] = 0  # TODO: Retrieve client timezone
        initial["schedule"] = schedule.copy()
        initial["control_schedule"] = schedule.copy()
        initial["resource_id"] = self.request.GET.get("resource_id")
        initial["unit_list"] = self.request.GET.getlist("unit_list")
        initial["trigger_type"] = self.request.GET.get("trigger_type")
        initial["trigger_parameters"] = json.loads(
            self.request.GET.get("trigger_parameters", "{}")
        )
        return initial

    @transaction.atomic
    def form_valid(
        self, form: forms.WialonNotificationCreationForm
    ) -> HttpResponse:
        try:
            customer = services.get_customer(self.request.user)
            token = services.get_wialon_token(self.request.user)
            if token is None:
                raise ValueError("Invalid/non-existent Wialon API token.")
            if customer is None:
                raise ValueError("Invalid/non-existent customer.")
            with WialonSession(token=token) as session:
                notification = form.save(commit=False)
                notification.customer = customer
                notification.text = notification.get_text()
                notification.actions = notification.get_actions()
                notification.trigger_type = form.cleaned_data["trigger_type"]
                notification.resource_id = form.cleaned_data["resource_id"]
                notification.unit_list = form.cleaned_data["unit_list"]
                notification.trigger_parameters = form.cleaned_data[
                    "trigger_parameters"
                ]

                api_response = notification.update_in_wialon("create", session)
                notification.wialon_id = int(api_response[0])
                notification.save()
                self.object = notification
                return HttpResponseRedirect(self.get_success_url())
        except WialonAPIError as e:
            form.add_error(
                None,
                ValidationError(
                    _("Whoops! '%(error_message)s'"),
                    code="invalid",
                    params={"error_message": str(e)},
                ),
            )
            return self.form_invalid(form=form)
        except ValueError as e:
            form.add_error(
                None,
                ValidationError(
                    _("Whoops! '%(error_message)s'"),
                    code="invalid",
                    params={"error_message": str(e)},
                ),
            )
            return self.form_invalid(form=form)


class WialonNotificationDetailView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, DetailView
):
    content_type = "text/html"
    http_method_names = ["get"]
    extra_context = {"title": "Notification Details"}
    model = models.WialonNotification
    partial_template_name = (
        "terminusgps_notifications/notifications/partials/_detail.html"
    )
    pk_url_kwarg = "notification_pk"
    template_name = "terminusgps_notifications/notifications/detail.html"

    def get_queryset(self) -> QuerySet:
        return super().get_queryset().filter(customer__user=self.request.user)


class WialonNotificationUpdateView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, UpdateView
):
    extra_context = {
        "message_tags": constants.WialonNotificationMessageTag.choices
    }
    content_type = "text/html"
    http_method_names = ["get", "post"]
    form_class = forms.WialonNotificationUpdateForm
    model = models.WialonNotification
    partial_template_name = (
        "terminusgps_notifications/notifications/partials/_update.html"
    )
    pk_url_kwarg = "notification_pk"
    template_name = "terminusgps_notifications/notifications/update.html"

    def get_queryset(self) -> QuerySet:
        return super().get_queryset().filter(customer__user=self.request.user)

    def form_valid(
        self, form: forms.WialonNotificationUpdateForm
    ) -> HttpResponse:
        notification = form.save(commit=True)
        token = services.get_wialon_token(notification.customer.user)
        if token and form.changed_data:
            try:
                with WialonSession(token=token) as session:
                    if "method" in form.changed_data:
                        notification.actions = notification.get_actions()
                    if "message" in form.changed_data:
                        notification.text = notification.get_text()
                    notification.update_in_wialon("update", session)
            except (ValueError, WialonAPIError) as e:
                form.add_error(
                    None,
                    ValidationError(
                        _("Whoops! %(error)s"),
                        code="invalid",
                        params={"error": str(e)},
                    ),
                )
                return self.form_invalid(form=form)
        return HttpResponseRedirect(notification.get_absolute_url())


class WialonNotificationDeleteView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, DeleteView
):
    content_type = "text/html"
    http_method_names = ["get", "post"]
    model = models.WialonNotification
    partial_template_name = (
        "terminusgps_notifications/notifications/partials/_delete.html"
    )
    pk_url_kwarg = "notification_pk"
    template_name = "terminusgps_notifications/notifications/delete.html"
    success_url = reverse_lazy("terminusgps_notifications:list notifications")

    def get_queryset(self) -> QuerySet:
        return super().get_queryset().filter(customer__user=self.request.user)

    def form_valid(self, form: Form) -> HttpResponse:
        try:
            token = services.get_wialon_token(self.request.user)
            if not token:
                raise ValueError("Invalid Wialon API token.")
            with WialonSession(token=token) as session:
                self.object.update_in_wialon("delete", session)
            return super().form_valid(form=form)
        except (ValueError, WialonAPIError) as e:
            logger.warning(e)
            return HttpResponse(status=406)


class WialonNotificationListView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, ListView
):
    allow_empty = True
    content_type = "text/html"
    http_method_names = ["get"]
    model = models.WialonNotification
    ordering = "name"
    partial_template_name = (
        "terminusgps_notifications/notifications/partials/_list.html"
    )
    template_name = "terminusgps_notifications/notifications/list.html"
    paginate_by = 8

    def setup(self, request: HttpRequest, *args, **kwargs) -> None:
        super().setup(request, *args, **kwargs)
        customer = (
            services.get_customer(self.request.user)
            if hasattr(self.request, "user")
            else None
        )
        login_params = (
            services.get_wialon_login_parameters(self.request.user.username)
            if hasattr(self.request, "user")
            else {}
        )
        has_token = (
            hasattr(customer, "token")
            if customer and customer is not None
            else False
        )
        has_subscription = (
            hasattr(customer, "subscription")
            if customer and customer is not None
            else False
        )
        self.customer = customer
        self.login_params = login_params
        self.has_token = has_token
        self.has_subscription = has_subscription

    def get_context_data(self, **kwargs) -> dict[str, typing.Any]:
        """Adds :py:attr:`customer`, :py:attr:`has_token`, :py:attr:`has_subscription`, :py:attr:`resource_id` and :py:attr:`login_params` to the view context."""
        resource_id = None
        if self.customer and self.has_token:
            session_key = f"{self.customer.pk}_resource_id"
            if self.request.session.get(session_key):
                resource_id = self.request.session[session_key]
            else:
                token = services.get_wialon_token(self.request.user)
                with WialonSession(token=token) as session:
                    resources = self.customer.get_resources_from_wialon(
                        session
                    )
                    resource_id = int(resources[0]["id"])
                    self.request.session[session_key] = resource_id
        context: dict[str, typing.Any] = super().get_context_data(**kwargs)
        context["customer"] = self.customer
        context["has_token"] = self.has_token
        context["login_params"] = self.login_params
        context["resource_id"] = resource_id
        return context

    def get_queryset(self) -> QuerySet:
        return (
            super()
            .get_queryset()
            .filter(customer__user=self.request.user)
            .order_by(self.get_ordering())
        )



from django.contrib import admin

from . import models


@admin.register(models.TerminusgpsNotificationsCustomer)
class TerminusgpsNotificationsCustomerAdmin(admin.ModelAdmin):
    list_display = [
        "user",
        "messages_count",
        "messages_max",
        "subscription__status",
    ]
    list_filter = ["subscription__status"]
    readonly_fields = [
        "tax",
        "grand_total",
        "messages_count",
        "messages_max",
        "subtotal",
    ]
    exclude = ["subscription"]
    fieldsets = [
        (None, {"fields": ["user", "date_format"]}),
        ("Messaging", {"fields": ["messages_count", "messages_max"]}),
        (
            "Pricing",
            {
                "fields": [
                    "tax_rate",
                    "subtotal_base",
                    "subtotal",
                    "tax",
                    "grand_total",
                ]
            },
        ),
    ]


@admin.register(models.WialonToken)
class WialonTokenAdmin(admin.ModelAdmin):
    list_display = ["customer"]
    exclude = ["name"]
    readonly_fields = ["flags"]


@admin.register(models.MessagePackage)
class MessagePackageAdmin(admin.ModelAdmin):
    list_display = ["customer", "price"]


@admin.register(models.WialonNotification)
class WialonNotificationAdmin(admin.ModelAdmin):
    list_display = ["name", "customer"]
    list_filter = ["customer"]
    readonly_fields = [
        "text",
        "wialon_id",
        "customer",
        "actions",
        "schedule",
        "control_schedule",
        "trigger_type",
        "trigger_parameters",
        "unit_list",
    ]



from django import forms
from django.conf import settings
from django.contrib.auth.forms import AuthenticationForm, BaseUserCreationForm
from django.core.validators import validate_email
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from terminusgps_payments.forms import PaymentProfileChoiceField
from terminusgps_payments.models import PaymentProfile

from terminusgps_notifications.models import MessagePackage

WIDGET_CSS_CLASS = (
    settings.WIDGET_CSS_CLASS
    if hasattr(settings, "WIDGET_CSS_CLASS")
    else "peer p-2 rounded border border-current bg-gray-50 dark:bg-gray-600 user-invalid:bg-red-50 user-invalid:text-red-600"
)


class MessagePackageCountField(forms.TypedChoiceField):
    coerce = int
    initial = 500

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.choices = [
            (500, _("500 Messages")),
            (1000, _("1000 messages")),
            (5000, _("5000 messages")),
        ]


class MessagePackageCreationForm(forms.ModelForm):
    payment = PaymentProfileChoiceField(
        queryset=PaymentProfile.objects.none(),
        widget=forms.widgets.Select(attrs={"class": WIDGET_CSS_CLASS}),
    )

    class Meta:
        model = MessagePackage
        fields = ["count"]
        field_classes = {"count": MessagePackageCountField}
        widgets = {
            "count": forms.widgets.Select(
                attrs={
                    "class": WIDGET_CSS_CLASS,
                    "hx-get": reverse_lazy(
                        "terminusgps_notifications:price packages"
                    ),
                    "hx-include": "this",
                    "hx-swap": "outerHTML",
                    "hx-target": "#id_price",
                    "hx-trigger": "load once, change",
                    "hx-indicator": "#price-indicator",
                }
            )
        }


class TerminusgpsNotificationsAuthenticationForm(AuthenticationForm):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.fields["username"].widget = forms.widgets.EmailInput(
            attrs={
                "class": WIDGET_CSS_CLASS,
                "placeholder": "email@terminusgps.com",
                "autofocus": True,
                "enterkeyhint": "next",
                "inputmode": "email",
            }
        )
        self.fields["password"].widget = forms.widgets.PasswordInput(
            attrs={
                "class": WIDGET_CSS_CLASS,
                "placeholder": "••••••••••••••••",
                "autocomplete": False,
                "inputmode": "text",
                "enterkeyhint": "done",
            }
        )


class TerminusgpsNotificationsRegistrationForm(BaseUserCreationForm):
    first_name = forms.CharField(
        help_text=_(
            "Required. 64 characters or fewer. Letters and digits only."
        ),
        label=_("First Name"),
        widget=forms.widgets.TextInput(
            attrs={
                "class": WIDGET_CSS_CLASS,
                "placeholder": "First",
                "enterkeyhint": "next",
            }
        ),
    )
    last_name = forms.CharField(
        help_text=_(
            "Required. 64 characters or fewer. Letters and digits only."
        ),
        label=_("Last Name"),
        widget=forms.widgets.TextInput(
            attrs={
                "class": WIDGET_CSS_CLASS,
                "placeholder": "Last",
                "enterkeyhint": "next",
            }
        ),
    )
    company_name = forms.CharField(
        label=_("Company Name"),
        help_text=_(
            "Optional. 64 characters or fewer. Letters and digits only."
        ),
        required=False,
        widget=forms.widgets.TextInput(
            attrs={
                "class": WIDGET_CSS_CLASS,
                "placeholder": "Terminus GPS",
                "enterkeyhint": "next",
            }
        ),
    )
    consent = forms.BooleanField(
        initial=False,
        widget=forms.widgets.CheckboxInput(
            attrs={"class": "accent-terminus-red-700"}
        ),
    )

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.fields["username"].validators.append(validate_email)
        self.fields["username"].widget = forms.widgets.TextInput(
            attrs={
                "class": WIDGET_CSS_CLASS,
                "placeholder": "email@terminusgps.com",
                "enterkeyhint": "next",
            }
        )
        self.fields["password1"].widget = forms.widgets.PasswordInput(
            attrs={
                "class": WIDGET_CSS_CLASS,
                "placeholder": "••••••••••••••••",
                "enterkeyhint": "next",
                "autocomplete": False,
            }
        )
        self.fields["password2"].widget = forms.widgets.PasswordInput(
            attrs={
                "class": WIDGET_CSS_CLASS,
                "placeholder": "••••••••••••••••",
                "enterkeyhint": "done",
                "autocomplete": False,
            }
        )



from django.urls import path

from . import views

app_name = "terminusgps_notifications"
urlpatterns = [
    path("", views.HomeView.as_view(), name="home"),
    path("register/", views.RegisterView.as_view(), name="register"),
    path("login/", views.LoginView.as_view(), name="login"),
    path("logout/", views.LogoutView.as_view(), name="logout"),
    path("navbar/", views.NavbarView.as_view(), name="navbar"),
    path("terms/", views.TermsView.as_view(), name="terms"),
    path("source/", views.SourceCodeView.as_view(), name="source code"),
    path("privacy/", views.PrivacyView.as_view(), name="privacy"),
    path("dashboard/", views.DashboardView.as_view(), name="dashboard"),
    path("account/", views.AccountView.as_view(), name="account"),
    path("stats/", views.CustomerStatsView.as_view(), name="stats"),
    path("contact/", views.ContactView.as_view(), name="contact"),
    path(
        "subscription/", views.SubscriptionView.as_view(), name="subscription"
    ),
    path(
        "subscriptions/create/",
        views.CustomerSubscriptionCreateView.as_view(),
        name="create subscriptions",
    ),
    path(
        "notifications/",
        views.NotificationsView.as_view(),
        name="notifications",
    ),
    path(
        "notifications/list/",
        views.WialonNotificationListView.as_view(),
        name="list notifications",
    ),
    path(
        "notifications/create/",
        views.WialonNotificationCreateView.as_view(),
        name="create notifications",
    ),
    path(
        "notifications/create/success/",
        views.WialonNotificationCreateSuccessView.as_view(),
        name="create notifications success",
    ),
    path(
        "notifications/<int:notification_pk>/detail/",
        views.WialonNotificationDetailView.as_view(),
        name="detail notifications",
    ),
    path(
        "notifications/<int:notification_pk>/update/",
        views.WialonNotificationUpdateView.as_view(),
        name="update notifications",
    ),
    path(
        "notifications/<int:notification_pk>/delete/",
        views.WialonNotificationDeleteView.as_view(),
        name="delete notifications",
    ),
    path(
        "notifications/units/form/",
        views.WialonNotificationUnitSelectFormView.as_view(),
        name="units form",
    ),
    path(
        "notifications/triggers/form/",
        views.WialonNotificationTriggerFormView.as_view(),
        name="triggers form",
    ),
    path(
        "notifications/triggers/parameters/form/",
        views.WialonNotificationTriggerParametersFormView.as_view(),
        name="trigger parameters form",
    ),
    path(
        "notifications/triggers/parameters/success/",
        views.WialonNotificationTriggerParametersSuccessView.as_view(),
        name="trigger parameters success",
    ),
    path(
        "packages/create/",
        views.MessagePackageCreateView.as_view(),
        name="create packages",
    ),
    path(
        "packages/list/",
        views.MessagePackageListView.as_view(),
        name="list packages",
    ),
    path(
        "packages/price/",
        views.MessagePackagePriceView.as_view(),
        name="price packages",
    ),
    path(
        "wialon/login/", views.WialonLoginView.as_view(), name="wialon login"
    ),
]



from urllib.parse import quote

from django.test import TestCase, override_settings

from terminusgps_notifications import services


class GetWialonRedirectUriTestCase(TestCase):
    @override_settings(DEBUG=False)
    def test_production_redirect_uri(self) -> None:
        """Fails if the function returns anything other than the production URL when DEBUG is false."""
        result = services.get_wialon_redirect_uri()
        self.assertTrue(result.startswith("https://api.terminusgps.com/"))
        self.assertIn("/account", result)

    @override_settings(DEBUG=True)
    def test_development_redirect_uri(self) -> None:
        """Fails if the function returns anything other than the development URL when DEBUG is true."""
        result = services.get_wialon_redirect_uri()
        self.assertTrue(result.startswith("http://127.0.0.1:8000/"))
        self.assertIn("/account", result)


class GetWialonLoginParametersTestCase(TestCase):
    def test_returns_correct_query_string(self) -> None:
        """Fails if the function returns anything other than a valid query string for the Wialon login page."""
        username = "testuser@terminusgps.com"
        result = services.get_wialon_login_parameters(username)
        self.assertIn("client_id=", result)
        self.assertIn("access_type=", result)
        self.assertIn("activation_time=", result)
        self.assertIn("duration=", result)
        self.assertIn("lang=", result)
        self.assertIn("flags=", result)
        self.assertIn(f"user={quote(username)}", result)
        self.assertIn("response_type=", result)

    @override_settings(WIALON_TOKEN_ACCESS_TYPE=1)
    def test_uses_access_type_from_settings(self) -> None:
        """Fails if the query string contains an access type other than 1."""
        username = "testuser@terminusgps.com"
        result = services.get_wialon_login_parameters(username)
        self.assertIn("access_type=1", result)



import logging
from typing import Any
from urllib.parse import urljoin

from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.urls import reverse
from django_tasks import task
from terminusgps.authorizenet.service import (
    AuthorizenetControllerExecutionError,
)
from terminusgps_payments.models import CustomerProfile, Subscription
from terminusgps_payments.services import AuthorizenetService

from terminusgps_notifications.models import TerminusgpsNotificationsCustomer

logger = logging.getLogger(__name__)
BASE_URL = "https://api.terminusgps.com/"


@task
def send_email_registration_confirmation(
    email_addr: str, first_name: str | None = None
) -> bool:
    template_name: str = (
        "terminusgps_notifications/emails/registration_confirmation.txt"
    )
    html_template_name: str = (
        "terminusgps_notifications/emails/registration_confirmation.html"
    )
    subject: str = "Terminus GPS Notifications - Account Registered"
    context: dict[str, str | None] = {
        "first_name": first_name,
        "link_homepage": urljoin(
            BASE_URL, reverse("terminusgps_notifications:home")
        ),
        "link_dashboard": urljoin(
            BASE_URL, reverse("terminusgps_notifications:dashboard")
        ),
        "link_account": urljoin(
            BASE_URL, reverse("terminusgps_notifications:account")
        ),
        "link_subscription": urljoin(
            BASE_URL, reverse("terminusgps_notifications:subscription")
        ),
        "link_notifications": urljoin(
            BASE_URL, reverse("terminusgps_notifications:notifications")
        ),
    }
    text_content: str = render_to_string(template_name, context=context)
    html_content: str = render_to_string(html_template_name, context=context)
    msg: EmailMultiAlternatives = EmailMultiAlternatives(
        subject=subject,
        body=text_content,
        to=[email_addr],
        bcc=[admin[1] for admin in settings.ADMINS],
    )
    msg.attach_alternative(html_content, "text/html")
    return bool(msg.send(fail_silently=True))


@task
def send_email_subscription_created(
    email_addr: str, first_name: str | None = None
) -> bool:
    template_name: str = (
        "terminusgps_notifications/emails/subscription_created.txt"
    )
    html_template_name: str = (
        "terminusgps_notifications/emails/subscription_created.html"
    )
    subject: str = "Terminus GPS Notifications - New Subscription"
    context: dict[str, Any] = {
        "first_name": first_name,
        "executions_max": 500,
        "link_homepage": urljoin(
            BASE_URL, reverse("terminusgps_notifications:home")
        ),
        "link_notifications": urljoin(
            BASE_URL, reverse("terminusgps_notifications:notifications")
        ),
    }
    text_content: str = render_to_string(template_name, context=context)
    html_content: str = render_to_string(html_template_name, context=context)
    msg: EmailMultiAlternatives = EmailMultiAlternatives(
        subject=subject,
        body=text_content,
        to=[email_addr],
        bcc=[admin[1] for admin in settings.ADMINS],
    )
    msg.attach_alternative(html_content, "text/html")
    return bool(msg.send(fail_silently=True))


@task
def send_email_subscription_updated(email_addr: str) -> bool:
    template_name: str = (
        "terminusgps_notifications/emails/subscription_updated.txt"
    )
    html_template_name: str = (
        "terminusgps_notifications/emails/subscription_updated.html"
    )
    subject: str = "Terminus GPS Notifications - Updated Subscription"
    context: dict[str, Any] = {}
    text_content: str = render_to_string(template_name, context=context)
    html_content: str = render_to_string(html_template_name, context=context)
    msg: EmailMultiAlternatives = EmailMultiAlternatives(
        subject=subject,
        body=text_content,
        to=[email_addr],
        bcc=[admin[1] for admin in settings.ADMINS],
    )
    msg.attach_alternative(html_content, "text/html")
    return bool(msg.send(fail_silently=True))


@task
def refresh_subscription_status(subscription_pk: int) -> str | None:
    """
    Refreshes and returns a subscription status from Authorizenet by pk.

    Returns :py:obj:`None` if the pk didn't point to a subscription.

    :param subscription_pk: A subscription primary key.
    :type subscription_pk: int
    :returns: The current subscription status from Authorizenet, if the subscription exists.
    :rtype: str | None

    """
    try:
        subscription = Subscription.objects.get(pk=subscription_pk)
        service = AuthorizenetService()
        new_status = str(service.get_subscription_status(subscription).status)
        subscription.status = new_status
        subscription.save(update_fields=["status"])
        return new_status
    except (
        Subscription.DoesNotExist,
        AuthorizenetControllerExecutionError,
    ) as e:
        logger.warning(e)


@task
def sync_authorizenet_profiles(customer_pk: int) -> None:
    """Retrieves customer profiles from Authorizenet and creates them locally."""
    try:
        customer = TerminusgpsNotificationsCustomer.objects.get(pk=customer_pk)
        cprofile = CustomerProfile.objects.get(user=customer.user)
        service = AuthorizenetService()
        anet_response = service.get_customer_profile(cprofile)
        print(f"{dir(anet_response) = }")
    except (
        TerminusgpsNotificationsCustomer.DoesNotExist,
        CustomerProfile.DoesNotExist,
    ) as e:
        logger.critical(e)



import typing

from django.contrib.auth.views import LoginView as LoginViewBase
from django.contrib.auth.views import LogoutView as LogoutViewBase
from django.db import transaction
from django.http import HttpResponse, HttpResponseRedirect
from django.urls import reverse, reverse_lazy
from django.utils.decorators import method_decorator
from django.views.decorators.cache import cache_control, cache_page
from django.views.generic import FormView, RedirectView, TemplateView
from terminusgps.mixins import HtmxTemplateResponseMixin

from terminusgps_notifications.forms import (
    TerminusgpsNotificationsAuthenticationForm,
    TerminusgpsNotificationsRegistrationForm,
)
from terminusgps_notifications.models import TerminusgpsNotificationsCustomer


class SourceCodeView(RedirectView):
    http_method_names = ["get"]
    permanent = True
    url = "https://github.com/terminusgps/terminusgps-notifications/"


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
@method_decorator(cache_control(private=True), name="dispatch")
class NavbarView(HtmxTemplateResponseMixin, TemplateView):
    content_type = "text/html"
    http_method_names = ["get"]
    partial_template_name = "terminusgps_notifications/partials/_navbar.html"
    template_name = "terminusgps_notifications/navbar.html"


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
class HomeView(HtmxTemplateResponseMixin, TemplateView):
    content_type = "text/html"
    extra_context = {
        "title": "Terminus GPS Notifications",
        "subtitle": "We know where ours are... do you?",
    }
    http_method_names = ["get"]
    partial_template_name = "terminusgps_notifications/partials/_home.html"
    template_name = "terminusgps_notifications/home.html"


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
class TermsView(HtmxTemplateResponseMixin, TemplateView):
    content_type = "text/html"
    http_method_names = ["get"]
    extra_context = {
        "title": "Terms & Conditions",
        "subtitle": "You agree to these by using Terminus GPS services",
    }
    template_name = "terminusgps_notifications/terms.html"
    partial_template_name = "terminusgps_notifications/partials/_terms.html"


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
class ContactView(HtmxTemplateResponseMixin, TemplateView):
    content_type = "text/html"
    http_method_names = ["get"]
    extra_context = {"title": "Contact", "subtitle": "Drop us a line"}
    template_name = "terminusgps_notifications/contact.html"
    partial_template_name = "terminusgps_notifications/partials/_contact.html"


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
class PrivacyView(HtmxTemplateResponseMixin, TemplateView):
    content_type = "text/html"
    extra_context = {
        "title": "Privacy Policy",
        "subtitle": "How we use your data",
    }
    http_method_names = ["get"]
    template_name = "terminusgps_notifications/privacy.html"
    partial_template_name = "terminusgps_notifications/partials/_privacy.html"


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
class LoginView(HtmxTemplateResponseMixin, LoginViewBase):
    content_type = "text/html"
    extra_context = {
        "title": "Login",
        "subtitle": "We know where ours are... do you?",
    }
    form_class = TerminusgpsNotificationsAuthenticationForm
    http_method_names = ["get", "post"]
    partial_template_name = "terminusgps_notifications/partials/_login.html"
    success_url = reverse_lazy("terminusgps_notifications:dashboard")
    template_name = "terminusgps_notifications/login.html"

    def get_initial(self, **kwargs) -> dict[str, typing.Any]:
        initial: dict[str, typing.Any] = super().get_initial(**kwargs)
        if username := self.request.GET.get("username"):
            initial["username"] = username
        return initial


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
class LogoutView(HtmxTemplateResponseMixin, LogoutViewBase):
    content_type = "text/html"
    http_method_names = ["get", "post"]
    template_name = "terminusgps_notifications/logged_out.html"
    partial_template_name = (
        "terminusgps_notifications/partials/_logged_out.html"
    )


@method_decorator(cache_page(timeout=60 * 15), name="dispatch")
class RegisterView(HtmxTemplateResponseMixin, FormView):
    content_type = "text/html"
    extra_context = {
        "title": "Register",
        "subtitle": "You'll know where yours are...",
    }
    form_class = TerminusgpsNotificationsRegistrationForm
    http_method_names = ["get", "post"]
    partial_template_name = "terminusgps_notifications/partials/_register.html"
    success_url = reverse_lazy("terminusgps_notifications:login")
    template_name = "terminusgps_notifications/register.html"

    @transaction.atomic
    def form_valid(
        self, form: TerminusgpsNotificationsRegistrationForm
    ) -> HttpResponse:
        user = form.save(commit=False)
        user.first_name = form.cleaned_data["first_name"]
        user.last_name = form.cleaned_data["last_name"]
        user.email = form.cleaned_data["username"]
        user.save()
        customer = TerminusgpsNotificationsCustomer(user=user)
        customer.company = form.cleaned_data["company_name"]
        customer.save()
        return HttpResponseRedirect(
            reverse(
                "terminusgps_notifications:login",
                query={"username": form.cleaned_data["username"]},
            )
        )



# Generated by Django 5.2.8 on 2025-11-13 20:20

import django.core.validators
import django.db.models.deletion
import django.db.models.expressions
import encrypted_field.fields
import terminusgps.wialon.flags
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('terminusgps_payments', '0002_alter_customerprofile_user'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='TerminusgpsNotificationsCustomer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('company', models.CharField(blank=True, default=None, max_length=64, null=True)),
                ('date_format', models.CharField(choices=[('%Y-%m-%d %H:%M:%S', 'YYYY-MM-DD HH:MM:SS'), ('%Y-%m-%d %H:%M', 'YYYY-MM-DD HH:MM')], default='%Y-%m-%d %H:%M:%S', help_text='Select a date format for notification messages.', max_length=24)),
                ('tax_rate', models.DecimalField(decimal_places=4, default=0.0825, help_text='Enter a tax rate as a decimal.', max_digits=9)),
                ('subtotal', models.DecimalField(decimal_places=2, default=60.0, help_text='Programatically generated customer subtotal amount (base + packages).', max_digits=9)),
                ('tax', models.GeneratedField(db_persist=True, expression=django.db.models.expressions.CombinedExpression(django.db.models.expressions.CombinedExpression(models.F('subtotal'), '*', django.db.models.expressions.CombinedExpression(models.F('tax_rate'), '+', models.Value(1))), '-', models.F('subtotal')), help_text='Automatically generated tax amount.', output_field=models.DecimalField(decimal_places=2, max_digits=9))),
                ('grand_total', models.GeneratedField(db_persist=True, expression=django.db.models.expressions.CombinedExpression(models.F('subtotal'), '*', django.db.models.expressions.CombinedExpression(models.F('tax_rate'), '+', models.Value(1))), help_text='Automatically generated grand total amount (subtotal+tax).', output_field=models.DecimalField(decimal_places=2, max_digits=9))),
                ('messages_max', models.PositiveIntegerField(default=500, verbose_name='maximum executions')),
                ('messages_max_base', models.PositiveIntegerField(default=500, verbose_name='maximum executions base')),
                ('messages_count', models.PositiveIntegerField(default=0, verbose_name='current executions')),
                ('subscription', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='terminusgps_notifications_customer', to='terminusgps_payments.subscription')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='terminusgps_notifications_customer', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'customer',
                'verbose_name_plural': 'customers',
            },
        ),
        migrations.CreateModel(
            name='MessagePackage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('price', models.DecimalField(decimal_places=2, default=40.0, max_digits=9)),
                ('count', models.IntegerField(default=0)),
                ('max', models.IntegerField(default=500)),
                ('customer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='packages', to='terminusgps_notifications.terminusgpsnotificationscustomer')),
            ],
            options={
                'verbose_name': 'message package',
                'verbose_name_plural': 'message packages',
            },
        ),
        migrations.CreateModel(
            name='WialonNotification',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Provide a memorable name.', max_length=64)),
                ('message', models.CharField(help_text='Enter a message.', max_length=1024)),
                ('method', models.CharField(choices=[('sms', 'SMS'), ('voice', 'Voice')], default='sms', help_text='Select a delivery method.', max_length=5)),
                ('activation_time', models.DateTimeField(blank=True, default=None, help_text='Provide a valid date and time in the format: YYYY-MM-DD HH:MM:SS. Leave this blank to activate immediately.', null=True)),
                ('deactivation_time', models.DateTimeField(blank=True, default=None, help_text='Provide a valid date and time in the format: YYYY-MM-DD HH:MM:SS. Leave this blank to never deactivate.', null=True)),
                ('max_alarms', models.PositiveIntegerField(default=0, help_text='Provide the maximum number of alarms. 0 = unlimited alarms.')),
                ('max_message_interval', models.PositiveIntegerField(choices=[(0, 'Any time'), (60, '1 minute'), (600, '10 minutes'), (1800, '30 minutes'), (3600, '1 hour'), (21600, '6 hours'), (43200, '12 hours'), (86400, '1 day'), (864000, '10 days')], default=3600, help_text='Select the maximum allowed time between messages.')),
                ('alarm_timeout', models.PositiveIntegerField(default=0, help_text='Provide the alarm timeout in seconds. 0 = never timeout.', validators=[django.core.validators.MaxValueValidator(1800)])),
                ('control_period', models.PositiveIntegerField(choices=[(0, 'Any time'), (60, 'Last minute'), (600, 'Last 10 minutes'), (3600, 'Last hour'), (86400, 'Last day')], default=3600, help_text='Select a control period relative to current time.')),
                ('min_duration_alarm', models.PositiveIntegerField(default=0, help_text='Provide the minimum duration of alarm state in seconds.', validators=[django.core.validators.MaxValueValidator(86400)])),
                ('min_duration_prev', models.PositiveIntegerField(default=0, validators=[django.core.validators.MaxValueValidator(86400)])),
                ('language', models.CharField(choices=[('en', 'English')], default='en', help_text='Select a valid language.', max_length=2)),
                ('flags', models.PositiveSmallIntegerField(choices=[(0, 'Trigger on first message'), (1, 'Trigger on every message'), (2, 'Disabled')], default=0)),
                ('timezone', models.IntegerField(default=0)),
                ('unit_list', models.JSONField(blank=True, default=list)),
                ('trigger_type', models.CharField(choices=[('geozone', 'Geofence'), ('address', 'Address'), ('speed', 'Speed'), ('alarm', 'Alarm'), ('digital_input', 'Digital input'), ('msg_param', 'Parameter in a message'), ('sensor_value', 'Sensor value'), ('outage', 'Connection loss'), ('interposition', 'Interposition of units'), ('msgs_counter', 'Excess of messages'), ('route_control', 'Route progress'), ('driver', 'Driver'), ('trailer', 'Trailer'), ('service_interals', 'Maintenance'), ('fuel_filling', 'Fuel filling/battery charge'), ('fuel_theft', 'Fuel drain/theft'), ('health_check', 'Health check')], default='sensor_value')),
                ('trigger_parameters', models.JSONField(blank=True, default=dict)),
                ('schedule', models.JSONField(blank=True, default=dict)),
                ('control_schedule', models.JSONField(blank=True, default=dict)),
                ('actions', models.JSONField(blank=True, default=dict)),
                ('text', models.CharField(blank=True, max_length=1024)),
                ('activations', models.PositiveIntegerField(default=0)),
                ('enabled', models.BooleanField(default=True)),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('wialon_id', models.PositiveBigIntegerField()),
                ('resource_id', models.PositiveBigIntegerField()),
                ('customer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='notifications', to='terminusgps_notifications.terminusgpsnotificationscustomer')),
            ],
            options={
                'verbose_name': 'wialon notification',
                'verbose_name_plural': 'wialon notifications',
            },
        ),
        migrations.CreateModel(
            name='WialonToken',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', encrypted_field.fields.EncryptedField(max_length=72)),
                ('flags', models.PositiveIntegerField(default=terminusgps.wialon.flags.TokenFlag['VIEW_ACCESS'] | terminusgps.wialon.flags.TokenFlag['MANAGE_NONSENSITIVE'] | terminusgps.wialon.flags.TokenFlag['MANAGE_SENSITIVE'])),
                ('customer', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='token', to='terminusgps_notifications.terminusgpsnotificationscustomer')),
            ],
        ),
    ]



from enum import StrEnum

from django.db import models
from django.utils.translation import gettext_lazy as _


# fmt: off
class WialonUnitSensorType(models.TextChoices):
    ANY = "", _("Any")
    # Mileage
    MILEAGE = "mileage", _("Mileage sensor")
    ODOMETER = "odometer", _("Relative odometer")
    # Digital
    IGNITION = "engine operation", _("Engine ignition sensor")
    ALARM = "alarm trigger", _("Alarm trigger")
    PRIVATE = "private mode", _("Private mode")
    RTMOTION = "real-time motion sensor", _("Real-time motion sensor")
    DIGITAL = "digital", _("Custom digital sensor")
    # Gauges
    VOLTAGE = "voltage", _("Voltage sensor")
    WEIGHT = "weight", _("Weight sensor")
    ACCELEROMETER = "accelerometer", _("Accelerometer")
    TEMPERATURE = "temperature", _("Temperature sensor")
    TEMPERATURE_COEF = "temperature coefficient", _("Temperature coefficient")
    # Engine
    ENGINE_RPM = "engine rpm", _("Engine revolution sensor")
    ENGINE_EFF = "engine efficiency", _("Engine efficiency sensor")
    ENGINE_HOURS_ABS = "engine hours", _("Absolute engine hours")
    ENGINE_HOURS_REL = "relative engine hours", _("Relative engine hours")
    # Fuel
    FUEL_CONS_IMP = "impulse fuel consumption", _("Impulse fuel consumption sensor")
    FUEL_CONS_ABS = "absolute fuel consumption", _("Absolute fuel consumption sensor")
    FUEL_CONS_INT = "instant fuel consumption", _("Instant fuel consumption sensor")
    FUEL_LEVEL = "fuel level", _("Fuel level sensor")
    FUEL_LEVEL_IMP = "fuel level impulse sensor", _("Impulse fuel level sensor")
    BATTERY_LEVEL = "battery level", _("Battery level sensor")
    # Other
    COUNTER = "counter", _("Counter sensor")
    CUSTOM = "custom", _("Custom sensor")
    DRIVER = "driver", _("Driver assignment")
    TRAILER = "trailer", _("Trailer assignment")
    TAG = "tag", _("Passenger sensor")


class WialonNotificationUpdateCallModeType(StrEnum):
    CREATE = "create"
    UPDATE = "update"
    DELETE = "delete"
    ENABLE = "enable"


class WialonNotificationTriggerType(models.TextChoices):
    GEOFENCE = "geozone", _("Geofence")
    ADDRESS = "address", _("Address")
    SPEED = "speed", _("Speed")
    ALARM = "alarm", _("Alarm")
    DIGITAL = "digital_input", _("Digital input")
    PARAMETER = "msg_param", _("Parameter in a message")
    SENSOR = "sensor_value", _("Sensor value")
    OUTAGE = "outage", _("Connection loss")
    INTERPOSITION = "interposition", _("Interposition of units")
    EXCESS = "msgs_counter", _("Excess of messages")
    ROUTE = "route_control", _("Route progress")
    DRIVER = "driver", _("Driver")
    TRAILER = "trailer", _("Trailer")
    MAINTENANCE = "service_interals", _("Maintenance")
    FUEL = "fuel_filling", _("Fuel filling/battery charge")
    FUEL_DRAIN = "fuel_theft", _("Fuel drain/theft")
    HEALTH = "health_check", _("Health check")


class WialonNotificationMessageTag(models.TextChoices):
    UNIT = "%UNIT%", _("Unit name")
    CURR_TIME = "%CURR_TIME%", _("Current date and time")
    LOCATION = "%LOCATION%", _("Location")
    LOCATOR_LINK = "%LOCATOR_LINK(60,T)%", _("Create locator link for the triggered unit (brackets indicate lifespan in minutes, T and G parameters to show tracks and geofences)")
    ZONE_MIN = "%ZONE_MIN%", _( "The smallest of geofences holding unit at the moment of notification")
    ZONE_ALL = "%ZONES_ALL%", _("All geofences holding unit at the moment of notification")
    UNIT_GROUP = "%UNIT_GROUP%", _("Groups containing triggered unit")
    SPEED = "%SPEED%", _("Unit speed at the moment of notification")
    POS_TIME = "%POS_TIME%", _("Date and time of the triggered message or the latest message with position in case the triggered message has no position")
    MSG_TIME = "%MSG_TIME%", _("Date and time of the triggered message")
    DRIVER = "%DRIVER%", _("Driver name")
    DRIVER_PHONE = "%DRIVER_PHONE%", _("Driver phone number")
    TRAILER = "%TRAILER%", _("Trailer name")
    SENSOR = "%SENSOR(*)%", _("Unit sensors and their values (indicate sensor mask in brackets)")
    ENGINE_HOURS = "%ENGINE_HOURS%", _("Engine hours at the moment of notification")
    MILEAGE = "%MILEAGE%", _("Mileage at the moment of notification")
    LAT = "%LAT%", _("Latitude at the moment of notification")
    LON = "%LON%", _("Longitude at the moment of notification")
    LATD = "%LATD%", _("Latitude at the moment of notification (without formatting)")
    LOND = "%LOND%", _("Longitude at the moment of notification (without formatting)")
    GOOGLE_LINK = "%GOOGLE_LINK%", _("Link to Google Maps with the position at the moment of notification")
    CUSTOM_FIELD = "%CUSTOM_FIELD(*)%", _("Custom field value from unit properties (indicate custom field name in brackets)")
    SENSOR_NAME = "%SENSOR_NAME%", _("Sensor name")
    SENSOR_VALUE = "%SENSOR_VALUE%", _("Triggered sensor value")
    TRIGGERED_SENSORS = "%TRIGGERED_SENSORS%", _("All triggered sensors and their values")
    VIDEO_LINK = "%VIDEO_LINK(480)%", _("Links to the video files saved for the triggered notification with the 'Save a video as a file' action. In parentheses, specify the validity period of the links in minutes, the maximum value is 2880")
    UNIT_ID = "%UNIT_ID%", _("Unit ID")
    MSG_TIME_INT = "%MSG_TIME_INT%", _("Time of triggered message in UNIX format")



import decimal
import typing

from authorizenet import apicontractsv1
from django.contrib.auth.mixins import LoginRequiredMixin
from django.core.exceptions import ValidationError
from django.db import transaction
from django.http import HttpRequest, HttpResponse, HttpResponseRedirect
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.views.generic import CreateView, ListView, TemplateView
from terminusgps.mixins import HtmxTemplateResponseMixin
from terminusgps_payments.models import PaymentProfile
from terminusgps_payments.services import AuthorizenetService

from terminusgps_notifications import forms, models, services


class MessagePackageCreateView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, CreateView
):
    content_type = "text/html"
    extra_context = {"title": "Create Package"}
    form_class = forms.MessagePackageCreationForm
    http_method_names = ["get", "post"]
    model = models.MessagePackage
    partial_template_name = (
        "terminusgps_notifications/packages/partials/_create.html"
    )
    template_name = "terminusgps_notifications/packages/create.html"

    def setup(self, request: HttpRequest, *args, **kwargs) -> None:
        super().setup(request, *args, **kwargs)
        self.customer = (
            services.get_customer(request.user)
            if hasattr(request, "user")
            else None
        )
        self.customer_profile = (
            services.get_customer_profile(request.user)
            if hasattr(request, "user")
            else None
        )

    def get_form(self, form_class=None) -> forms.MessagePackageCreationForm:
        form = super().get_form(form_class=form_class)
        form.fields["payment"].empty_label = None
        form.fields["payment"].queryset = (
            PaymentProfile.objects.for_user(self.request.user)
            if hasattr(self.request, "user")
            else PaymentProfile.objects.none()
        )
        return form

    @transaction.atomic
    def form_valid(
        self, form: forms.MessagePackageCreationForm
    ) -> HttpResponse:
        if self.customer is None:
            form.add_error(
                None,
                ValidationError(
                    _(
                        "Whoops! You don't have a customer associated with your account. Please try again later."
                    ),
                    code="invalid",
                ),
            )
            return self.form_invalid(form=form)
        if self.customer_profile is None:
            form.add_error(
                None,
                ValidationError(
                    _(
                        "Whoops! You don't have a customer profile associated with your account. Please try again later."
                    ),
                    code="invalid",
                ),
            )
            return self.form_invalid(form=form)
        if int(form.cleaned_data["count"]) % 500 != 0:
            form.add_error(
                "count",
                ValidationError(
                    _(
                        "Whoops! You can only purchase messages in 500 message blocks. Please enter a number divisible by 500 and try again."
                    ),
                    code="invalid",
                ),
            )
            return self.form_invalid(form=form)
        package = models.MessagePackage()
        package.customer = self.customer
        package.count = int(form.cleaned_data["count"])
        package.price = round(
            decimal.Decimal(package.count / 500) * decimal.Decimal("40.00"),
            ndigits=2,
        )

        line_item = apicontractsv1.lineItemType()
        line_item.itemId = "1"
        line_item.name = f"x{package.count} messages package"
        line_item.description = (
            "Message package for Terminus GPS Notifications platform"
        )
        line_item.quantity = "1"
        line_item.unitPrice = package.price
        line_items = apicontractsv1.ArrayOfLineItem()
        line_items.lineItem.append(line_item)

        service = AuthorizenetService()
        service.charge_customer_profile(
            customer_profile=self.customer_profile,
            payment_profile=form.cleaned_data["payment"],
            amount=package.price,
            line_items=line_items,
        )
        package.save()
        return HttpResponseRedirect(
            reverse("terminusgps_notifications:list packages")
        )


class MessagePackageListView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, ListView
):
    content_type = "text/html"
    extra_context = {"title": "List Packages"}
    http_method_names = ["get"]
    model = models.MessagePackage
    partial_template_name = (
        "terminusgps_notifications/packages/partials/_list.html"
    )
    template_name = "terminusgps_notifications/packages/list.html"


class MessagePackagePriceView(
    LoginRequiredMixin, HtmxTemplateResponseMixin, TemplateView
):
    content_type = "text/html"
    extra_context = {"title": "Package Price"}
    http_method_names = ["get"]
    template_name = "terminusgps_notifications/packages/price.html"
    partial_template_name = (
        "terminusgps_notifications/packages/partials/_price.html"
    )

    def get(self, request: HttpRequest, *args, **kwargs) -> HttpResponse:
        if not hasattr(request, "GET") or not request.GET.get("count"):
            return HttpResponse(status=406)
        elif (
            isinstance(request.GET["count"], str)
            and not request.GET["count"].isdigit()
        ):
            return HttpResponse(status=406)
        self.count = int(request.GET["count"])
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs) -> dict[str, typing.Any]:
        context: dict[str, typing.Any] = super().get_context_data(**kwargs)
        context["price"] = (
            decimal.Decimal(self.count / 500) * decimal.Decimal("40.00")
            if self.count % 500 == 0
            else None
        )
        return context



from urllib.parse import urlencode, urljoin

from django.conf import settings
from django.contrib.auth.models import AbstractBaseUser
from django.urls import reverse
from terminusgps_payments.models import CustomerProfile

from terminusgps_notifications.models import TerminusgpsNotificationsCustomer


def get_customer(
    user: AbstractBaseUser,
) -> TerminusgpsNotificationsCustomer | None:
    """
    Returns the :py:obj:`~terminusgps_notifications.models.TerminusgpsNotificationsCustomer` for a user.

    :param user: A Django user.
    :type user: ~django.contrib.auth.models.AbstractBaseUser
    :returns: A customer object, if the user has one.
    :rtype: ~terminusgps_notifications.models.TerminusgpsNotificationsCustomer | None

    """
    if hasattr(user, "terminusgps_notifications_customer"):
        return getattr(user, "terminusgps_notifications_customer")


def get_customer_profile(user: AbstractBaseUser) -> CustomerProfile | None:
    """
    Returns the :py:obj:`~terminusgps_payments.models.CustomerProfile` for a user.

    :param user: A Django user.
    :type user: ~django.contrib.auth.models.AbstractBaseUser
    :returns: A customer profile object, if the user has one.
    :rtype: ~terminusgps_payments.models.CustomerProfile | None

    """
    if hasattr(user, "customer_profile"):
        return getattr(user, "customer_profile")


def get_wialon_token(user: AbstractBaseUser) -> str | None:
    """
    Returns the :py:obj:`~terminusgps_notifications.models.WialonToken` for a user.

    :param user: A Django user.
    :type user: ~django.contrib.auth.models.AbstractBaseUser
    :returns: A Wialon API token, if the user has one.
    :rtype: str | None

    """
    customer = get_customer(user)
    if customer is not None and hasattr(customer, "token"):
        return getattr(customer, "token").name


def get_wialon_redirect_uri() -> str:
    """Returns the redirect (callback) URI for Wialon authentication."""
    return urljoin(
        "https://api.terminusgps.com/"
        if not settings.DEBUG
        else "http://127.0.0.1:8000/",
        reverse("terminusgps_notifications:account"),
    )


def get_wialon_login_parameters(username: str) -> str:
    """
    Returns a query string of path parameters to be added to the Wialon authentication request.

    :param username: A username to autofill into the authentication form.
    :type username: str
    :returns: A query string.
    :rtype: str

    """
    return urlencode(
        {
            "client_id": "Terminus GPS Notifications",
            "access_type": settings.WIALON_TOKEN_ACCESS_TYPE,
            "activation_time": 0,
            "duration": 0,
            "lang": "en",
            "flags": 0x1,
            "user": username,
            "response_type": "token",
            "redirect_uri": get_wialon_redirect_uri(),
        }
    )



import logging
import typing
import urllib.parse

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.validators import MaxValueValidator
from django.db import models
from django.db.models import F
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from encrypted_field import EncryptedField
from terminusgps.wialon.flags import DataFlag
from terminusgps.wialon.session import WialonSession

from terminusgps_notifications.constants import (
    WialonNotificationTriggerType,
    WialonNotificationUpdateCallModeType,
)

logger = logging.getLogger(__name__)


class TerminusgpsNotificationsCustomer(models.Model):
    """A Terminus GPS Notifications customer."""

    user = models.OneToOneField(
        get_user_model(),
        on_delete=models.CASCADE,
        related_name="terminusgps_notifications_customer",
    )
    """Django user."""
    company = models.CharField(
        max_length=64, null=True, blank=True, default=None
    )
    """Company name."""
    date_format = models.CharField(
        choices=[
            ("%Y-%m-%d %H:%M:%S", "YYYY-MM-DD HH:MM:SS"),
            ("%Y-%m-%d %H:%M", "YYYY-MM-DD HH:MM"),
        ],
        default="%Y-%m-%d %H:%M:%S",
        help_text="Select a date format for notification messages.",
        max_length=24,
    )
    """Date format for notifications."""
    tax_rate = models.DecimalField(
        max_digits=9,
        decimal_places=4,
        default=0.0825,
        help_text="Enter a tax rate as a decimal.",
    )
    """Subscription tax rate."""
    subtotal = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=60.00,
        help_text="Programatically generated customer subtotal amount (base + packages).",
    )
    """Subscription current subtotal."""
    tax = models.GeneratedField(
        expression=(F("subtotal") * (F("tax_rate") + 1)) - F("subtotal"),
        output_field=models.DecimalField(max_digits=9, decimal_places=2),
        db_persist=True,
        help_text="Automatically generated tax amount.",
    )
    """Subscription tax total."""
    grand_total = models.GeneratedField(
        expression=F("subtotal") * (F("tax_rate") + 1),
        output_field=models.DecimalField(max_digits=9, decimal_places=2),
        db_persist=True,
        help_text="Automatically generated grand total amount (subtotal+tax).",
    )
    """Subscription grand total (subtotal + tax)."""
    subscription = models.ForeignKey(
        "terminusgps_payments.Subscription",
        on_delete=models.SET_NULL,
        related_name="terminusgps_notifications_customer",
        null=True,
        blank=True,
        default=None,
    )
    """Associated subscription."""

    messages_max = models.PositiveIntegerField(
        verbose_name="maximum executions", default=500
    )
    """Maximum number of notification executions in a single period."""
    messages_max_base = models.PositiveIntegerField(
        verbose_name="maximum executions base", default=500
    )
    """Maximum base number of notification executions in a single period."""
    messages_count = models.PositiveIntegerField(
        verbose_name="current executions", default=0
    )
    """Current number of notification executions this period."""

    class Meta:
        verbose_name = _("customer")
        verbose_name_plural = _("customers")

    def __str__(self) -> str:
        """Returns the customer user's username."""
        return str(self.user.username)

    def get_units_from_wialon(
        self,
        resource_id: str | int,
        session: WialonSession,
        items_type: str = "avl_unit",
        force: bool = False,
        flags: int = DataFlag.UNIT_BASE,
        start: int = 0,
        end: int = 0,
    ) -> list[dict[str, typing.Any]]:
        """
        Returns a list of of customer Wialon unit dictionaries from the Wialon API.

        Default Wialon unit dictionary format (flags=1):

        +------------+---------------+---------------------------+
        | key        | type          | desc                      |
        +============+===============+===========================+
        | ``"mu"``   | :py:obj:`int` | Measurement system        |
        +------------+---------------+---------------------------+
        | ``"nm"``   | :py:obj:`str` | Unit name                 |
        +------------+---------------+---------------------------+
        | ``"cls"``  | :py:obj:`int` | Superclass ID: 'avl_unit' |
        +------------+---------------+---------------------------+
        | ``"id"``   | :py:obj:`int` | Unit ID                   |
        +------------+---------------+---------------------------+
        | ``"uacl"`` | :py:obj:`int` | User's access rights      |
        +------------+---------------+---------------------------+

        :param force: Whether to force a Wialon API call instead of using a cached response. Default is :py:obj:`False` (use cache).
        :type force: bool
        :param flags: Response flags. Default is ``1``.
        :type flags: int
        :param start: Start index. Default is ``0``.
        :type start: int
        :param end: End index. Default is ``0`` (no limit).
        :type end: int
        :raises ValueError: If ``resource_id`` was a string and contained non-digit characters.
        :returns: A list of Wialon unit dictionaries.
        :rtype: list[dict[str, ~typing.Any]]

        """
        if isinstance(resource_id, str) and not resource_id.isdigit():
            raise ValueError(
                f"resource_id can only contain digits, got '{resource_id}'."
            )
        return session.wialon_api.core_search_items(
            **{
                "spec": {
                    "itemsType": items_type,
                    "propName": "sys_billing_account_guid,sys_name",
                    "propValueMask": f"{resource_id},*",
                    "sortType": "sys_name",
                    "propType": "property,property",
                },
                "force": int(force),
                "flags": flags,
                "from": start,
                "to": end,
            }
        ).get("items", [])

    def get_resources_from_wialon(
        self,
        session: WialonSession,
        force: bool = False,
        flags: int = DataFlag.RESOURCE_BASE,
        start: int = 0,
        end: int = 0,
    ) -> list[dict[str, typing.Any]]:
        """
        Returns a list of of customer Wialon resource dictionaries from the Wialon API.

        Default Wialon resource dictionary format (flags=1):

        +------------+---------------+-------------------------------+
        | key        | type          | desc                          |
        +============+===============+===============================+
        | ``"mu"``   | :py:obj:`int` | Measurement system            |
        +------------+---------------+-------------------------------+
        | ``"nm"``   | :py:obj:`str` | Resource name                 |
        +------------+---------------+-------------------------------+
        | ``"cls"``  | :py:obj:`int` | Superclass ID: 'avl_resource' |
        +------------+---------------+-------------------------------+
        | ``"id"``   | :py:obj:`int` | Resource ID                   |
        +------------+---------------+-------------------------------+
        | ``"uacl"`` | :py:obj:`int` | User's access rights          |
        +------------+---------------+-------------------------------+

        :param force: Whether to force a Wialon API call instead of using a cached response. Default is :py:obj:`False` (use cache).
        :type force: bool
        :param flags: Response flags. Default is ``1``.
        :type flags: int
        :param start: Start index. Default is ``0``.
        :type start: int
        :param end: End index. Default is ``0`` (no limit).
        :type end: int
        :returns: A list of Wialon resource dictionaries.
        :rtype: list[dict[str, ~typing.Any]]

        """
        return session.wialon_api.core_search_items(
            **{
                "spec": {
                    "itemsType": "avl_resource",
                    "propName": "sys_name",
                    "propValueMask": "*",
                    "sortType": "sys_name",
                    "propType": "property",
                },
                "force": int(force),
                "flags": flags,
                "from": start,
                "to": end,
            }
        ).get("items", [])


class MessagePackage(models.Model):
    price = models.DecimalField(max_digits=9, decimal_places=2, default=40.00)
    """Message package price."""
    count = models.IntegerField(default=0)
    """Message package current execution count."""
    max = models.IntegerField(default=500)
    """Message package maximum allowed executions."""
    customer = models.ForeignKey(
        "terminusgps_notifications.TerminusgpsNotificationsCustomer",
        on_delete=models.CASCADE,
        related_name="packages",
    )
    """Associated customer."""

    class Meta:
        verbose_name = _("message package")
        verbose_name_plural = _("message packages")

    def __str__(self) -> str:
        return f"MessagePackage #{self.pk}"


class WialonToken(models.Model):
    """Wialon API token."""

    customer = models.OneToOneField(
        "terminusgps_notifications.TerminusgpsNotificationsCustomer",
        on_delete=models.CASCADE,
        related_name="token",
    )
    """Associated customer."""

    name = EncryptedField(max_length=72)
    """Wialon API token name."""
    flags = models.PositiveIntegerField(
        default=settings.WIALON_TOKEN_ACCESS_TYPE
    )
    """Wialon token flags."""

    def __str__(self) -> str:
        """Returns '<customer email>'s WialonToken'."""
        return f"{self.customer}'s WialonToken"


class WialonNotification(models.Model):
    """Wialon notification."""

    class WialonNotificationMethod(models.TextChoices):
        """Wialon notification method."""

        SMS = "sms", _("SMS")
        VOICE = "voice", _("Voice")

    name = models.CharField(
        max_length=64, help_text="Provide a memorable name."
    )
    """Notification name."""
    message = models.CharField(max_length=1024, help_text="Enter a message.")
    """Notification message."""
    method = models.CharField(
        choices=WialonNotificationMethod.choices,
        default=WialonNotificationMethod.SMS,
        help_text="Select a delivery method.",
        max_length=5,
    )
    """Notification method."""

    activation_time = models.DateTimeField(
        blank=True,
        default=None,
        help_text="Provide a valid date and time in the format: YYYY-MM-DD HH:MM:SS. Leave this blank to activate immediately.",
        null=True,
    )
    """Activation date/time."""
    deactivation_time = models.DateTimeField(
        blank=True,
        default=None,
        help_text="Provide a valid date and time in the format: YYYY-MM-DD HH:MM:SS. Leave this blank to never deactivate.",
        null=True,
    )
    """Deactivation date/time."""
    max_alarms = models.PositiveIntegerField(
        default=0,
        help_text="Provide the maximum number of alarms. 0 = unlimited alarms.",
    )
    """Maximum number of alarms (0 = unlimited)."""
    max_message_interval = models.PositiveIntegerField(
        default=3600,
        choices=[
            (0, _("Any time")),
            (60, _("1 minute")),
            (600, _("10 minutes")),
            (1800, _("30 minutes")),
            (3600, _("1 hour")),
            (21600, _("6 hours")),
            (43200, _("12 hours")),
            (86400, _("1 day")),
            (864000, _("10 days")),
        ],
        help_text="Select the maximum allowed time between messages.",
    )
    """Max time interval between messages."""
    alarm_timeout = models.PositiveIntegerField(
        default=0,
        validators=[MaxValueValidator(1800)],
        help_text="Provide the alarm timeout in seconds. 0 = never timeout.",
    )
    """Alarm timeout in seconds. Max is ``1800`` (30 minutes in seconds)."""
    control_period = models.PositiveIntegerField(
        default=3600,
        choices=[
            (0, _("Any time")),
            (60, _("Last minute")),
            (600, _("Last 10 minutes")),
            (3600, _("Last hour")),
            (86400, _("Last day")),
        ],
        help_text="Select a control period relative to current time.",
    )
    """Control period relative to current time in seconds."""
    min_duration_alarm = models.PositiveIntegerField(
        default=0,
        validators=[MaxValueValidator(86400)],
        help_text="Provide the minimum duration of alarm state in seconds.",
    )
    """Minimum duration of alarm state in seconds. Max is ``86400`` (1 day)."""
    min_duration_prev = models.PositiveIntegerField(
        default=0, validators=[MaxValueValidator(86400)]
    )
    """Minimum duration of previous state in seconds. Max is ``86400`` (1 day)."""
    language = models.CharField(
        max_length=2,
        default="en",
        choices=[("en", _("English"))],
        help_text="Select a valid language.",
    )
    """2-letter language code."""
    flags = models.PositiveSmallIntegerField(
        default=0,
        choices=[
            (0, _("Trigger on first message")),
            (1, _("Trigger on every message")),
            (2, _("Disabled")),
        ],
    )
    """Flags."""

    timezone = models.IntegerField(default=0)
    """Timezone."""
    unit_list = models.JSONField(blank=True, default=list)
    """List of Wialon units."""
    trigger_type = models.CharField(
        choices=WialonNotificationTriggerType.choices,
        default=WialonNotificationTriggerType.SENSOR,
    )
    """Trigger type."""
    trigger_parameters = models.JSONField(blank=True, default=dict)
    """Trigger parameters."""
    schedule = models.JSONField(blank=True, default=dict)
    """Schedule."""
    control_schedule = models.JSONField(blank=True, default=dict)
    """Control schedule."""
    actions = models.JSONField(blank=True, default=dict)
    """Actions."""
    text = models.CharField(max_length=1024, blank=True)
    """Text."""
    activations = models.PositiveIntegerField(default=0)
    """Activation count."""
    enabled = models.BooleanField(default=True)
    """Whether the notification is enabled in Wialon."""
    date_created = models.DateTimeField(auto_now_add=True)
    """Date created."""

    wialon_id = models.PositiveBigIntegerField()
    """Wialon id."""
    resource_id = models.PositiveBigIntegerField()
    """Resource id."""
    customer = models.ForeignKey(
        "terminusgps_notifications.TerminusgpsNotificationsCustomer",
        on_delete=models.CASCADE,
        related_name="notifications",
    )
    """Associated customer."""

    class Meta:
        verbose_name = _("wialon notification")
        verbose_name_plural = _("wialon notifications")

    def __str__(self) -> str:
        """Returns the notification name."""
        return str(self.name)

    def get_absolute_url(self) -> str:
        """Returns a URL pointing to the notification's detail view."""
        return reverse(
            "terminusgps_notifications:detail notifications",
            kwargs={"notification_pk": self.pk},
        )

    def get_text(self) -> str:
        """Returns the notification text (txt)."""
        user_id: int = self.customer.user.pk
        message: str = self.message
        return f"unit_id=%UNIT_ID%&user_id={user_id}&message={message}"

    def get_actions(self) -> list[dict[str, typing.Any]]:
        """Returns a list of notification actions (act)."""
        return [
            {
                "t": "push_messages",
                "p": {
                    "url": urllib.parse.urljoin(
                        "https://api.terminusgps.com/",
                        f"/v3/notify/{self.method}/",
                    ),
                    "get": 1,  # 1 = GET request, 2 = POST request
                },
            }
        ]

    def get_data_from_wialon(
        self, session: WialonSession
    ) -> dict[str, typing.Any]:
        """
        Returns the notification data from Wialon using the Wialon API.

        Notification data format:

        +----------------+----------------+-----------------------------------------------+
        | key            | type           | desc                                          |
        +================+================+===============================================+
        | ``"id"``       | :py:obj:`int`  | Notification ID                               |
        +----------------+----------------+-----------------------------------------------+
        | ``"n"``        | :py:obj:`str`  | Notification name                             |
        +----------------+----------------+-----------------------------------------------+
        | ``"txt"``      | :py:obj:`int`  | Notification text                             |
        +----------------+----------------+-----------------------------------------------+
        | ``"ta"``       | :py:obj:`int`  | Activation time (UNIX timestamp)              |
        +----------------+----------------+-----------------------------------------------+
        | ``"td"``       | :py:obj:`int`  | Deactivation time (UNIX timestamp)            |
        +----------------+----------------+-----------------------------------------------+
        | ``"ma"``       | :py:obj:`int`  | Maximum number of alarms (0 = unlimited)      |
        +----------------+----------------+-----------------------------------------------+
        | ``"mmtd"``     | :py:obj:`int`  | Maximum time interval between messages (sec)  |
        +----------------+----------------+-----------------------------------------------+
        | ``"cdt"``      | :py:obj:`int`  | Alarm timeout (sec)                           |
        +----------------+----------------+-----------------------------------------------+
        | ``"mast"``     | :py:obj:`int`  | Minimum duration of the alarm state (sec)     |
        +----------------+----------------+-----------------------------------------------+
        | ``"mpst"``     | :py:obj:`int`  | Minimum duration of previous state (sec)      |
        +----------------+----------------+-----------------------------------------------+
        | ``"cp"``       | :py:obj:`int`  | Control period relative to current time (sec) |
        +----------------+----------------+-----------------------------------------------+
        | ``"fl"``       | :py:obj:`int`  | Notification flags                            |
        +----------------+----------------+-----------------------------------------------+
        | ``"tz"``       | :py:obj:`int`  | Notification timezone                         |
        +----------------+----------------+-----------------------------------------------+
        | ``"la"``       | :py:obj:`str`  | Notification language code                    |
        +----------------+----------------+-----------------------------------------------+
        | ``"ac"``       | :py:obj:`int`  | Alarms count                                  |
        +----------------+----------------+-----------------------------------------------+
        | ``"d"``        | :py:obj:`str`  | Notification description                      |
        +----------------+----------------+-----------------------------------------------+
        | ``"sch"``      | :py:obj:`dict` | Notification schedule (see below)             |
        +----------------+----------------+-----------------------------------------------+
        | ``"ctrl_sch"`` | :py:obj:`dict` | Notification control schedule (see below)     |
        +----------------+----------------+-----------------------------------------------+
        | ``"un"``       | :py:obj:`list` | List of unit/unit group IDs                   |
        +----------------+----------------+-----------------------------------------------+
        | ``"act"``      | :py:obj:`list` | List of notification actions (see below)      |
        +----------------+----------------+-----------------------------------------------+
        | ``"trg"``      | :py:obj:`dict` | Notification trigger (see below)              |
        +----------------+----------------+-----------------------------------------------+
        | ``"ct"``       | :py:obj:`int`  | Creation time (UNIX timestamp)                |
        +----------------+----------------+-----------------------------------------------+
        | ``"mt"``       | :py:obj:`int`  | Last modification time (UNIX timestamp)       |
        +----------------+----------------+-----------------------------------------------+

        Notification schedule/control schedule format:

        +----------+---------------+------------------------------------------------------------------+
        | key      | type          | desc                                                             |
        +==========+===============+==================================================================+
        | ``"f1"`` | :py:obj:`int` | Beginning of interval 1 (minutes from midnight)                  |
        +----------+---------------+------------------------------------------------------------------+
        | ``"f2"`` | :py:obj:`int` | Beginning of interval 2 (minutes from midnight)                  |
        +----------+---------------+------------------------------------------------------------------+
        | ``"t1"`` | :py:obj:`int` | End of interval 1 (minutes from midnight)                        |
        +----------+---------------+------------------------------------------------------------------+
        | ``"t2"`` | :py:obj:`int` | End of interval 2 (minutes from midnight)                        |
        +----------+---------------+------------------------------------------------------------------+
        | ``"m"``  | :py:obj:`int` | Mask of the days of the month (1: 2:\ sup:`0`, 31: 2\ :sup:`30`) |
        +----------+---------------+------------------------------------------------------------------+
        | ``"y"``  | :py:obj:`int` | Mask of months (Jan: 2\ :sup:`0`, Dec: 2:\ sup:`11`)             |
        +----------+---------------+------------------------------------------------------------------+
        | ``"w"``  | :py:obj:`int` | Mask of days of the week (Mon: 2\ :sup:`0`, Sun: 2\ :sup:`6`)    |
        +----------+---------------+------------------------------------------------------------------+
        | ``"f"``  | :py:obj:`int` | Schedule flags                                                   |
        +----------+---------------+------------------------------------------------------------------+

        Notification `action <https://wialon-help.link/bb04a9a5>`_ format (each item in the ``act`` list):

        +----------+-----------------+-------------------+
        | key      | type            | desc              |
        +==========+=================+===================+
        | ``"t"``  | :py:obj:`str`   | Action type       |
        +----------+-----------------+-------------------+
        | ``"p"``  | :py:obj:`dict`  | Action parameters |
        +----------+-----------------+-------------------+

        Notification `trigger <https://wialon-help.link/9d54585d>`_ format:

        +----------+-----------------+--------------------+
        | key      | type            | desc               |
        +==========+=================+====================+
        | ``"t"``  | :py:obj:`str`   | Trigger type       |
        +----------+-----------------+--------------------+
        | ``"p"``  | :py:obj:`dict`  | Trigger parameters |
        +----------+-----------------+--------------------+

        :param session: A valid Wialon API session.
        :type session: ~terminusgps.wialon.session.WialonSession
        :raises WialonAPIError: If something went wrong calling the Wialon API.
        :returns: The notification data from Wialon.
        :rtype: dict[str, ~typing.Any]

        """
        return session.wialon_api.resource_get_notification_data(
            **{"itemId": self.resource_id, "col": [self.wialon_id]}
        )

    def update_in_wialon(
        self,
        call_mode: WialonNotificationUpdateCallModeType,
        session: WialonSession,
    ) -> dict[str, typing.Any]:
        """
        Updates the notification in Wialon.

        :param call_mode: Call mode to use when calling ``resource/update_notification``.
        :type call_mode: ~terminusgps_notifications.constants.WialonNotificationUpdateCallMode
        :param session: A valid Wialon API session.
        :type session: ~terminusgps.wialon.session.WialonSession
        :raises WialonAPIError: If something went wrong calling the Wialon API.
        :returns: A dictionary of notification data.
        :rtype: dict[str, ~typing.Any]

        """
        params = self.get_wialon_parameters(call_mode=call_mode)
        return session.wialon_api.resource_update_notification(**params)

    def get_wialon_parameters(self, call_mode: str) -> dict[str, typing.Any]:
        """
        Returns parameters for Wialon notification API calls.

        :param call_mode: A Wialon API update notification call mode.
        :type call_mode: str
        :returns: A dictionary of Wialon API update notification parameters.
        :rtype: dict[str, ~typing.Any]

        """
        return {
            "itemId": self.resource_id,
            "id": 0 if call_mode == "create" else self.wialon_id,
            "callMode": call_mode,
            "n": self.name,
            "txt": self.text,
            "ta": 0,  # TODO: Convert ta/td attributes to timestamps
            "td": 0,
            "ma": self.max_alarms,
            "mmtd": self.max_message_interval,
            "cdt": self.alarm_timeout,
            "mast": self.min_duration_alarm,
            "mpst": self.min_duration_prev,
            "cp": self.control_period,
            "fl": self.flags,
            "la": self.language,
            "tz": self.timezone,
            "un": self.unit_list,
            "trg": {"t": self.trigger_type, "p": self.trigger_parameters},
            "act": self.actions,
            "sch": self.schedule,
            "ctrl_sch": self.control_schedule,
        }
